@@ -86,11 +86,15 @@ def Coverage( dir_file_bam, thres_mapq = 200 ) :
 
     return dict_refname_to_cov
     
-def Variant_Calling( r, dict_genome, set_mut_filtered = None, return_corrected_read_sequence = False ) :
-    ''' perform variant calling of a single aligned read (pysam read object) using a given genome
+def Variant_Calling( r, dict_genome, set_mut_filtered = None, return_corrected_read_sequence = False, flag_ignore_indel = False, flag_return_as_string = True, flag_return_matched = False ) :
+    ''' # 2021-08-27 20:46:12 
+    perform variant calling of a single aligned read (pysam read object) using a given genome
     return a list of mutations and corrected read sequence
     'set_mut_filtered' : only consider mutations in the given 'set_mut_filtered'. only valid when 'return_corrected_read_sequence' = True
     'return_corrected_read_sequence' : return aligned read sequence after hard-clipping and filtering variants using 'set_mut_filtered'
+    'flag_ignore_indel' : ignore insertion and deletion mutation calls
+    'flag_return_as_string' : return a called mutation as a string in the following format: f"{reference_name}:{start}_{str_operation}_{mutation}". If set to False, return the called mutation as a tuple (reference_name, start, str_operation, mutation, quality)
+    'flag_return_matched' : return 'id_mut' even when there is no mutation. the 'str_operation' for matched base is 'M'
     '''
     pos_read, pos_ref = 0, 0 # current position in the extracted reference sequence and the current read
     str_seq_ref = dict_genome[ r.reference_name ][ r.reference_start : r.reference_start + r.alen ] # retrieve a part of the reference sequence where the current read was aligned
@@ -101,13 +105,16 @@ def Variant_Calling( r, dict_genome, set_mut_filtered = None, return_corrected_r
         for n_bases, str_oper in NGS_CIGAR_Iterate_a_CIGAR_String( r.cigarstring ) :
             if str_oper == 'M' :
                 for _ in range( n_bases ) :
-                    str_base_read, str_base_ref = r.seq[ pos_read ], str_seq_ref[ pos_ref ]
+                    str_base_read, str_base_ref, str_qual_read = r.seq[ pos_read ], str_seq_ref[ pos_ref ], r.query_qualities[ pos_read ]
                     str_base_read_corrected = str_base_ref # default corrected read base = ref
                     if str_base_read != str_base_ref :
-                        id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" # compose id_mut, convert 0-based coordinate to 1-based coordinate
+                        id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'X', str_base_read, str_qual_read ) # compose id_mut, convert 0-based coordinate to 1-based coordinate when calling a mutation as a string
                         if set_mut_filtered is None or id_mut in set_mut_filtered :
                             l_mut.append( id_mut )
                             str_base_read_corrected = str_base_read
+                    elif flag_return_matched :
+                        id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_M_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'M', str_base_read, str_qual_read ) # compose id_mut, convert 0-based coordinate to 1-based coordinate when calling a mutation as a string
+                        l_mut.append( id_mut )
                     str_seq_corrected_read += str_base_read_corrected # add corrected read 
                     pos_ref += 1
                     pos_read += 1
@@ -116,14 +123,14 @@ def Variant_Calling( r, dict_genome, set_mut_filtered = None, return_corrected_r
             elif str_oper == 'S' :
                 pos_read += n_bases
             elif str_oper == 'I' :
-                id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_I_{r.seq[ pos_read : pos_read + n_bases ]}" # compose id_mut, 0-based coordinate to 1-based coordinate
-                if set_mut_filtered is None or id_mut in set_mut_filtered :
+                id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_I_{r.seq[ pos_read : pos_read + n_bases ]}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'I', r.seq[ pos_read : pos_read + n_bases ], r.query_qualities[ pos_read : pos_read + n_bases ] ) # compose id_mut, 0-based coordinate to 1-based coordinate when returning id_mut as a string
+                if not flag_ignore_indel and ( set_mut_filtered is None or id_mut in set_mut_filtered ) :
                     l_mut.append( id_mut ) 
                     str_seq_corrected_read += r.seq[ pos_read : pos_read + n_bases ] # add inserted sequence (if insertion is valid)
                 pos_read += n_bases
             elif str_oper == 'D' :
-                id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_D_{n_bases}" # compose id_mut, 0-based coordinate to 1-based coordinate
-                if set_mut_filtered is None or id_mut in set_mut_filtered :
+                id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_D_{n_bases}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'D', n_bases, None ) # compose id_mut, 0-based coordinate to 1-based coordinate when returning id_mut as a string
+                if not flag_ignore_indel and ( set_mut_filtered is None or id_mut in set_mut_filtered ) :
                     l_mut.append( id_mut ) 
                 else :
                     str_seq_corrected_read += str_seq_ref[ pos_ref : pos_ref + n_bases ] # add deleted reference sequence (if deletion is invalid)
@@ -133,14 +140,18 @@ def Variant_Calling( r, dict_genome, set_mut_filtered = None, return_corrected_r
             elif str_oper == 'P' :
                 pass
             elif str_oper == '=' :
+                if flag_return_matched :
+                    str_base_read, str_qual_read = r.seq[ pos_read ], r.query_qualities[ pos_read ]
+                    id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_M_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'M', str_base_read, str_qual_read ) # compose id_mut, convert 0-based coordinate to 1-based coordinate when calling a mutation as a string
+                    l_mut.append( id_mut )
                 str_seq_corrected_read += r.seq[ pos_read : pos_read + n_bases ] # add read sequences
                 pos_ref += n_bases
                 pos_read += n_bases
             elif str_oper == 'X' :
                 for _ in range( n_bases ) :
-                    str_base_read, str_base_ref = r.seq[ pos_read ], str_seq_ref[ pos_ref ]
+                    str_base_read, str_base_ref, str_qual_read = r.seq[ pos_read ], str_seq_ref[ pos_ref ], r.query_qualities[ pos_read ]
                     str_base_read_corrected = str_base_ref # default corrected read base = ref
-                    id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" # compose id_mut, 0-based coordinate to 1-based coordinate
+                    id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'X', str_base_read, str_qual_read ) # compose id_mut, 0-based coordinate to 1-based coordinate when returning id_mut as a string
                     if set_mut_filtered is None or id_mut in set_mut_filtered :
                         l_mut.append( id_mut ) 
                         str_base_read_corrected = str_base_read
@@ -153,9 +164,11 @@ def Variant_Calling( r, dict_genome, set_mut_filtered = None, return_corrected_r
         for n_bases, str_oper in NGS_CIGAR_Iterate_a_CIGAR_String( r.cigarstring ) :
             if str_oper == 'M' :
                 for _ in range( n_bases ) :
-                    str_base_read, str_base_ref = r.seq[ pos_read ], str_seq_ref[ pos_ref ]
+                    str_base_read, str_base_ref, str_qual_read = r.seq[ pos_read ], str_seq_ref[ pos_ref ], r.query_qualities[ pos_read ]
                     if str_base_read != str_base_ref :
-                        l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" ) # 0-based coordinate to 1-based coordinate
+                        l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'X', str_base_read, str_qual_read ) ) # 0-based coordinate to 1-based coordinate when returning id_mut as a string
+                    elif flag_return_matched :
+                        l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_M_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'M', str_base_read, str_qual_read ) ) # 0-based coordinate to 1-based coordinate when returning id_mut as a string
                     pos_ref += 1
                     pos_read += 1
             elif str_oper == 'N' :
@@ -163,23 +176,32 @@ def Variant_Calling( r, dict_genome, set_mut_filtered = None, return_corrected_r
             elif str_oper == 'S' :
                 pos_read += n_bases
             elif str_oper == 'I' :
-                l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_I_{r.seq[ pos_read : pos_read + n_bases ]}" ) # 0-based coordinate to 1-based coordinate
+                if not flag_ignore_indel :
+                    l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_I_{r.seq[ pos_read : pos_read + n_bases ]}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'I', r.seq[ pos_read : pos_read + n_bases ], r.query_qualities[ pos_read : pos_read + n_bases ] ) ) # 0-based coordinate to 1-based coordinate when returning id_mut as a string
                 pos_read += n_bases
             elif str_oper == 'D' :
-                l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_D_{n_bases}" ) # 0-based coordinate to 1-based coordinate
+                if not flag_ignore_indel :
+                    l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_D_{n_bases}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'D', n_bases, None ) ) # 0-based coordinate to 1-based coordinate when returning id_mut as a string
                 pos_ref += n_bases
             elif str_oper == 'H' :
                 pass
             elif str_oper == 'P' :
                 pass
             elif str_oper == '=' :
+                if flag_return_matched :
+                    str_base_read, str_qual_read = r.seq[ pos_read ], r.query_qualities[ pos_read ]
+                    id_mut = f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_M_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'M', str_base_read, str_qual_read ) # compose id_mut, convert 0-based coordinate to 1-based coordinate when calling a mutation as a string
+                    l_mut.append( id_mut )
                 pos_ref += n_bases
                 pos_read += n_bases
             elif str_oper == 'X' :
                 for _ in range( n_bases ) :
-                    str_base_read = r.seq[ pos_read ]
-                    l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" ) # 0-based coordinate to 1-based coordinate
+                    str_base_read, str_qual_read = r.seq[ pos_read ], r.query_qualities[ pos_read ]
+                    l_mut.append( f"{r.reference_name}:{r.reference_start + 1 + pos_ref}_X_{str_base_read}" if flag_return_as_string else ( r.reference_name, r.reference_start + pos_ref, 'X', str_base_read, str_qual_read ) ) # 0-based coordinate to 1-based coordinate when returning id_mut as a string
                     pos_ref += 1
                     pos_read += 1
         return l_mut
 
+
+
+
